#include "DataType.h"
#include<iterator>
#include<optional>

std::optional<int> readInt(std::istream& in) {
	int i;
	if(in>>i)
	{
		return { i };
	}
	return {};
}

Node::Node(int p_node_id,const std::array<double,3>& p_coors,bool p_vertex)
	:node_id(p_node_id)
	,coors(p_coors)
	,vertex(p_vertex)
{}

Node::Node(const Node& p_node)
	:node_id(p_node.node_id)
	,coors(p_node.coors)
	,vertex(p_node.vertex)
{}

std::istream& operator>>(std::istream& p_in, Node& p_node)
{
	p_in >> p_node.node_id;
	p_in >> get<0>(p_node.coors) >> get<1>(p_node.coors) >> get<2>(p_node.coors);
	p_in >> p_node.vertex;
	return p_in;
}

std::ostream& operator <<(std::ostream& p_out, const Node& p_node)
{
	p_out << p_node.node_id;
	p_out << get<0>(p_node.coors) << get<1>(p_node.coors) << get<2>(p_node.coors);
	p_out << p_node.vertex;
	return p_out;
}

std::istream& operator>>(std::istream& p_in, Edge& p_edge)
{
	p_in >> p_edge.nodes.first >> p_edge.nodes.second;
	return p_in;
}

std::ostream& operator<<(std::ostream& p_out, Edge& p_edge)
{
	p_out << "[" << p_edge.nodes.first << "," << p_edge.nodes.second << "]";
	return p_out;
}

std::istream& operator >>(std::istream& p_in, Finite_Element& p_Finite_Element)
{
	p_in >> p_Finite_Element.element_id >> p_Finite_Element.material_id;
	std::optional<int> p_node_id;
	p_node_id = readInt(p_in);
	while (p_node_id) {
		p_Finite_Element.node_id.push_back(*p_node_id);
		p_node_id = readInt(p_in);
	}
	return p_in;
}

std::ostream& operator<<(std::ostream& p_out, const Finite_Element& p_Finite_Element)
{
	p_out << p_Finite_Element.element_id << std::endl << p_Finite_Element.material_id << std::endl;
	p_out << "[";
	for (const auto& elem : p_Finite_Element.node_id)
		p_out << elem << "  ";
	p_out << "]";
	return p_out;
}

Finite_Element::Finite_Element(int p_element_id, int p_material_id, const std::vector<int>& p_node_id)
	:element_id(p_element_id)
	,material_id(p_material_id)
	,node_id(p_node_id)
{}

std::istream& operator>>(std::istream& p_in, Boundary_Finite_Element& p_Boundary_Finite_Element)
{
	p_in >> p_Boundary_Finite_Element.surface_id >> p_Boundary_Finite_Element.element_id;
	std::optional<int> p_node_id;
	p_node_id = readInt(p_in);
	while (p_node_id) {
		p_Boundary_Finite_Element.node_id.push_back(*p_node_id);
		p_node_id = readInt(p_in);
	}
	return p_in;
}

std::ostream& operator<<(std::ostream& p_out, const Boundary_Finite_Element& p_Boundary_Finite_Element)
{
	p_out << p_Boundary_Finite_Element.surface_id << std::endl << p_Boundary_Finite_Element.element_id << std::endl;
	p_out << "[";
	for (const auto& elem : p_Boundary_Finite_Element.node_id)
		p_out << elem << "  ";
	p_out << "]";
	return p_out;
}

Boundary_Finite_Element::Boundary_Finite_Element(int p_element_id, int p_bound_id, const std::vector<int>& p_node_id)
	:element_id(p_element_id)
	,surface_id(p_bound_id)
	,node_id(p_node_id)
{}

Edge::Edge(int p_id_first_node, int p_id_second_node)
	:nodes(std::make_pair(p_id_first_node,p_id_second_node))
{}

Edge Edge::operator=(const Edge& p_edge)
{
	Edge  t_edge;
	t_edge.nodes = p_edge.nodes;
	return t_edge;
}

bool Edge::operator==(const Edge& p_edge) const
{
	return (nodes.first == p_edge.nodes.first) && (nodes.second == p_edge.nodes.second);
}

Edge Edge::Create_Edge(int p_first_node_id, int p_second_node_id)
{
	return Edge(p_first_node_id, p_second_node_id);
}

void Edge::Update_Middle_Node(int p_middle_node)
{
	middle_node = p_middle_node;
}